#include "Display.h"

Display::Display()
{
    fd = 0;
};

int Display::ascii(int code)
{
    char ascii[5];
    sprintf(ascii, "~A%02x", code);
    print(ascii);
    return 0;
}

int Display::init(char *dev)
{
    int modelines = 0;
    struct termios settings;
    
    if ((fd = open(dev, O_RDWR | O_NOCTTY)) < 0) return 2;

    bzero(&settings, sizeof(settings));
    
    if (!isatty(fd)) return 3; //Fehler, da das Dev kein Terminal ist
    
    settings.c_iflag = ( IGNBRK | IGNPAR | IXON | IXOFF | ISTRIP | ECHO );
    settings.c_oflag = ( OPOST | ICANON | ISIG | ECHO ) & ~OLCUC;
    settings.c_cflag = ( CS8 | CSTOPB | CREAD | CLOCAL | CRTSCTS );
    settings.c_lflag = CRTSCTS;
    settings.c_cc[VTIME] = 1;
    settings.c_cc[VMIN] = 0;

    if (cfsetspeed(&settings, B38400) != 0) return 4;  //Fehler beim Setzen der Geschwindigkeit

    tcflush(fd, TCIOFLUSH);
    
    if (tcsetattr(fd, TCSANOW, &settings) == -1) return 5; //Fehler beim Setzen der Attribute
    
    if (ioctl(fd, TIOCMGET, &modelines) == -1) return 6; //Fehler bei ioctl TIOCMGET
    
    modelines &= ~TIOCM_RTS;
    if (ioctl(fd, TIOCMSET, &modelines) == -1) return 7; //Fehler bei ioctl TIOCMSET

    clear();
    return 0;
};

int Display::setLEDs(enum LED Led1, enum LED Led2, enum LED Led3)
{
    char leds[13];
    sprintf(leds, "~L0%d~L1%d~L2%d", Led1, Led2, Led3);
    print(leds);
    return 0;
};

int Display::getKeys()
{
    if (fd <= 0) return -1;
    char buf[2];
    int nbyte = 0;
    int retval = 0;
    
    nbyte = read(fd, &buf, 1);
    if ((nbyte == 1) && (buf[0] == 'T'))
    {
	nbyte = read(fd, &buf, 1);
	retval =  buf[0] - '0';
    } 
    else
    {
	retval = 0;
    }
    fflush(0);
    return retval;
};

int Display::quit()
{
    return (close(fd));
};

int Display::clear()
{
    clearScreen();
    clearLEDs();
    return 0;
};

int Display::clearLEDs()
{
    setLEDs(LED_OFF, LED_OFF, LED_OFF);
    return 0;
};

int Display::clearScreen()
{
    print("~C");
    return 0;
};

int Display::gotoxy(int x, int y)
{
    char pos[6];
    sprintf(pos, "~P%1d%02d", y - 1, x - 1);
    print(pos);
    return 0;
};

int Display::print(char *text)
{
    if (fd <= 0) return 2;
    struct timeval tv;
    tv.tv_sec = 1;
    tv.tv_usec = 100;
//    tv.tv_usec = 500;
    usleep(tv.tv_usec);
    write(fd, text, strlen(text));
    return 0;
};

int Display::defUC(int num, char *pattern)
{
    char uc[20];
    sprintf(uc, "~U%d%s", num, pattern);
    print(uc);
    return 0;
};

int Display::home()
{
    print("~H");
    return 0;
};

int Display::setLight(enum LIGHT stat)
{
    char light[4];
    sprintf(light, "~B%d", stat);
    print(light);
    return 0;
};

int Display::center(int y, char *text)
{
    gotoxy(10 - (strlen(text) + 1) / 2 + 1, y);
    print(text);
    return 0;
};
